# -*- coding:utf-8 -*-

import six
from django.db import models
from knob.dt import local_now
from .fields import DataField


@six.python_2_unicode_compatible
class Basket(models.Model):
    class Meta:
        pass

    uri = models.CharField(max_length=128, primary_key=True, unique=True)
    name = models.CharField(max_length=128, default='', null=True, blank=True)
    desc = models.TextField(default='', null=True, blank=True)

    create_dt = models.DateTimeField(default=local_now)
    update_dt = models.DateTimeField(default=local_now)

    def __str__(self):
        return u'[{}] {}'.format(self.uri, self.name)

    @property
    def varnames(self):
        return self.variables.values_list('name', flat=True)

    def get(self, varname, default=None):
        try:
            var = self.variables.get(name=varname)
            return var.data
        except Variable.DoesNotExist:
            return default

    def set(self, varname, data=None):
        if isinstance(varname, dict):
            vars = varname
        else:
            vars = {varname: data}

        now = local_now()
        for vn, d in six.iteritems(vars):
            var, created = self.variables.update_or_create(basket=self, name=vn, defaults={"data": d, "update_dt": now})

        self.update_dt = now
        self.save(update_fields=['update_dt'])


@six.python_2_unicode_compatible
class Variable(models.Model):
    class Meta:
        unique_together = [('basket', 'name')]

    basket = models.ForeignKey(Basket, on_delete=models.CASCADE, related_name='variables')
    name = models.CharField(max_length=32)
    desc = models.TextField(default='', null=True, blank=True)
    data = DataField(null=True, blank=True, compress=True)

    create_dt = models.DateTimeField(default=local_now)
    update_dt = models.DateTimeField(default=local_now)

    def __str__(self):
        return u"[{}] {}".format(self.basket.uri, self.name)

    def set_data(self, data):
        self.data = data
        self.update_dt = local_now()
        self.save(update_fields=['data', 'update_dt'])



# -*- coding:utf-8 -*-

import six
import numpy as np
from django.db import models
from django.test import TestCase
from rest_framework import serializers
from djsci.fields import *
from djsci.drf_fields import *
from hypothesis import given
import hypothesis.strategies as st
from hypothesis.extra.numpy import arrays, array_dtypes, scalar_dtypes, array_shapes


class TestingArrayModel(models.Model):
    arr = NumpyArrayField()


class TestingArraySerializer(serializers.ModelSerializer):
    class Meta:
        model = TestingArrayModel
        fields = ['id', 'arr']

    arr = DRFNumpyArrayField()


class FieldTestCase(TestCase):
    def test_arr(self):
        for i in range(10):
            arr = arrays(dtype='f8', shape=array_shapes()).example()
            m = TestingArrayModel(arr=arr)
            # m.save()
            # m = TestingArrayModel.objects.get(pk=m.id)
            six.print_(arr, TestingArraySerializer(m).data)
            self.assertEqual(np.allclose(arr, m.arr, equal_nan=True), True)


# Generated by Django 2.1.1 on 2018-09-17 05:58

from django.db import migrations, models
import django.db.models.deletion
import djsci.fields
import knob.dt


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Basket',
            fields=[
                ('uri', models.CharField(max_length=128, primary_key=True, serialize=False, unique=True)),
                ('name', models.CharField(blank=True, default='', max_length=128, null=True)),
                ('desc', models.TextField(blank=True, default='', null=True)),
                ('create_dt', models.DateTimeField(default=knob.dt.local_now)),
                ('update_dt', models.DateTimeField(default=knob.dt.local_now)),
            ],
        ),
        migrations.CreateModel(
            name='Variable',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=32)),
                ('desc', models.TextField(blank=True, default='', null=True)),
                ('data', djsci.fields.DataField(blank=True, editable=False, null=True)),
                ('create_dt', models.DateTimeField(default=knob.dt.local_now)),
                ('update_dt', models.DateTimeField(default=knob.dt.local_now)),
                ('basket', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='variables', to='djsci.Basket')),
            ],
        ),
        migrations.AlterUniqueTogether(
            name='variable',
            unique_together={('basket', 'name')},
        ),
    ]



# -*- coding:utf-8 -*-
# Django RestFramework (DRF) serializer fields

from rest_framework.fields import Field as DRFField
import numpy as np
try:
    import pandas as pd
    HAS_PANDAS = True
except ImportError:
    HAS_PANDAS = False
from .serialize_utils import *


class DRFDataField(DRFField):
    def to_representation(self, value):
        if isinstance(value, np.ndarray):
            return serialize_numpy_array(value)
        else:
            raise NotImplementedError(u'Data type {} is not supported by DRFDataField.'.format(type(value)))

    def to_internal_value(self, data):
        if isinstance(data, dict):
            data_type = data.get('type')
            if data_type == 'NumpyArray':
                return deserialize_numpy_array(data)
            else:
                raise NotImplementedError(u'Not supported: {}'.format(data_type))

        raise NotImplementedError(u'Not supported: {}'.format(type(data)))


class DRFNumpyArrayField(DRFField):
    def to_representation(self, value):
        return serialize_numpy_array(value)

    def to_internal_value(self, data):
        return deserialize_numpy_array(data)


class DRFPandasDataFrameField(DRFField):
    def to_representation(self, value):
        if not HAS_PANDAS:
            raise RuntimeError("Pandas not installed!")

    def to_internal_value(self, data):
        if not HAS_PANDAS:
            raise RuntimeError("Pandas not installed!")


class DRFPandasSeriesField(DRFField):
    def to_representation(self, value):
        if not HAS_PANDAS:
            raise RuntimeError("Pandas not installed!")

    def to_internal_value(self, data):
        if not HAS_PANDAS:
            raise RuntimeError("Pandas not installed!")




# -*- coding:utf-8 -*-

import numpy as np


def serialize_numpy_array(arr):
    result = {
        "type": "NumpyArray",
        "dtype": arr.dtype.str,
        "shape": list(arr.shape),
        "data": arr.tolist()
    }
    return result


def deserialize_numpy_array(data):
    if isinstance(data, list):
        return np.array(data)
    elif isinstance(data, dict):
        return np.array(data['data'], dtype=data.get('dtype'))
    else:
        raise NotImplementedError(u'Cannot deserialize data type {} into numpy array'.format(type(data)))



# -*- coding:utf-8 -*-

from django.contrib import admin
from .models import *


@admin.register(Basket)
class BasketAdmin(admin.ModelAdmin):
    list_display = ['uri', 'name', 'desc', 'varnames_', 'create_dt', 'update_dt']
    list_filter = ['create_dt', 'update_dt']
    search_fields = ['uri', 'name', 'desc']

    def varnames_(self, obj):
        return u', '.join(obj.varnames)


@admin.register(Variable)
class VariableAdmin(admin.ModelAdmin):
    list_display = ['id', 'basket', 'name', 'desc', 'create_dt', 'update_dt']
    list_filter = ['create_dt', 'update_dt']
    search_fields = ['id', 'name', 'desc',
                     'basket__uri', 'basket__name']



# -*- coding:utf-8 -*-

from picklefield.fields import PickledObjectField


class DataField(PickledObjectField):
    """A Generic Data Field"""
    # TODO: add type attr to this field
    pass


class NumpyArrayField(PickledObjectField):
    # TODO: customize value_to_string(), etc
    pass


class PandasSeriesField(PickledObjectField):
    # TODO: customize value_to_string(), etc
    pass


class PandasDataFrameField(PickledObjectField):
    # TODO: customize value_to_string(), etc
    pass